/*
 * File:   main.c
 * Author: RichardvdK
 *
 * Created on 26 july 2019, 10:25
 */


#include <xc.h>

#include <stdio.h>
#include "main.h"
#include "i2c.h"
#include "mpu6050.h"
#include "sh1106.h"
#include "ms5611.h"
#include "hmc5883l.h"
#include <math.h>
#include <stdlib.h>
#include "extender.h"
#include "common.h"
#include "memory.h"
#include "demo.h"

void delay_ms(unsigned int ms) {
    unsigned int i = 0;

    for (i = 0; i < ms; i++) {
        __delay_ms(1);
    }
}



UG_GUI gui_sh1106;

void main(void) {

    OSCTUNEbits.PLLEN = 1;

    OSCCONbits.IRCF = 0b111;
    ANSELA = 0x00;
    ANSELB = 0x00;
    ANSELC = 0x00;

    TRISB = 0x01;
    TRISC = 0x18;
    
    // Set pull-up for RB6
    INTCON2bits.RBPU = 0;
    TRISBbits.RB6 = 1;
    WPUB = 0b01000000;
    

    INTCON2bits.RBPU = 0;

    SSP1CON1bits.SSPM = 0b1000;
    SSP1CON1bits.SSPEN = 1;
    SSP1ADD = 39;

    //SSP1ADD = 159;

    //    SSP1ADD = 4;
    //SSP1ADD = 19;
    SSP1CON2bits.ACKDT = 0;


    delay_ms(100);

    lcd_init();
    UG_Init(&gui_sh1106, pset, 128, 64);
    UG_SelectGUI(&gui_sh1106);
    UG_FillScreen(C_BLACK);
    UG_FontSelect(&FONT_6X10);
    UG_PutString(7, 25, "Init sensors...");
    put_sh1106();
    delay_ms(10);

    //NOP();

    mpu6050_init();
    delay_ms(10);

    hmc5883l_init();
    delay_ms(10);

    ms5611_init();
    delay_ms(10);


    extender_init(device1);
    extender_init(device2);
    extender_init(device3);




    unsigned int C1 = get_c(PROM_Read_C1);
    __delay_ms(10);
    unsigned int C2 = get_c(PROM_Read_C2);
    __delay_ms(10);
    unsigned int C3 = get_c(PROM_Read_C3);
    __delay_ms(10);
    unsigned int C4 = get_c(PROM_Read_C4);
    __delay_ms(10);
    unsigned int C5 = get_c(PROM_Read_C5);
    __delay_ms(10);
    unsigned int C6 = get_c(PROM_Read_C6);









    int x_offset = 0;
    int y_offset = 0;

    signed char decli_deg = 0;
    char decli_sec = 0;



    x_offset = x_offset + DATAEE_ReadByte(0x00) << 8;
    delay_ms(10);
    x_offset = x_offset + DATAEE_ReadByte(0x01);
    delay_ms(10);
    y_offset = y_offset + DATAEE_ReadByte(0x10) << 8;
    delay_ms(10);
    y_offset = y_offset + DATAEE_ReadByte(0x11);
    delay_ms(10);
    decli_deg = DATAEE_ReadByte(0x20);
    delay_ms(10);
    decli_sec = DATAEE_ReadByte(0x21);
    delay_ms(10);

    if (x_offset == 0xffff) {
        x_offset = 0;
        y_offset = 0;
    }

    if (decli_deg == 0xff) {
        decli_deg = 0;
        decli_sec = 0;
    }


    float rad = ((decli_deg + (decli_sec / 60.0)) / 180.0) * M_PI;


    demo();


    clearDisplay();
    put_sh1106();

    

    char count_temp = 120;
    char count_press = 50;


    float source = 1013.25;


    long long TEMP = 0;
    long long SENS = 0;
    long long OFF = 0;
    long long P = 0;


    while (1) {


        if (count_temp == 120) {

            unsigned long long D2 = get_ms5611_temperature(x_offset, y_offset, rad);
            long long dT = D2 - ((unsigned long) C5 << 8);
            TEMP = 2000 + ((dT * C6) >> 23);
            OFF = ((unsigned long) C2 << 16) + ((C4 * dT) >> 7);
            SENS = ((unsigned long) C1 << 15) + ((C3 * dT) >> 8);

            long long T2 = 0;
            long long OFF2 = 0;
            long long SENS2 = 0;

            if (TEMP < 2000) {
                T2 = dT * dT / (unsigned long) 0x80000000;
                OFF2 = 5 * ((TEMP - 2000) * (TEMP - 2000)) / 2;
                SENS2 = 5 * ((TEMP - 2000) * (TEMP - 2000)) / 4;
            }

            if (TEMP < -1500) {
                OFF2 = OFF2 + 7 * ((TEMP + 1500) * (TEMP + 1500));
                SENS2 = SENS2 + 11 * ((TEMP + 1500) * (TEMP + 1500)) / 2;
            }

            TEMP = TEMP - T2;
            OFF = OFF - OFF2;
            SENS = SENS - SENS2;


            char press_temp[25];

            if (PORTBbits.RB6 == 1) {
                sprintf(press_temp, "%.2f", (float) TEMP / 100.0);
                UG_PutString(15, 5, press_temp);
                UG_FontSelect(&FONT_4X6);
                UG_PutString(75, 4, "O");
                UG_FontSelect(&FONT_6X10);
                UG_PutString(81, 5, "C");

            } else {
                float temp2 = (float) TEMP / 100.0 * 1.8 + 32.0;
                sprintf(press_temp, "%.2f", temp2);
                UG_PutString(15, 5, press_temp);
                UG_FontSelect(&FONT_6X10);
                UG_PutString(75, 4, "F");
            }


            count_temp = 0;

        }




        if (count_press == 50) {

            unsigned long long avg_press[7];
            for (int i = 0; i < 7; i++) {
                unsigned long long value = get_ms5611_pressure(x_offset, y_offset, rad);
                avg_press[i] = value;
            }



            unsigned long long inner, outer, temp;

            for (outer = 0; outer < 7 - 1; outer++) {
                for (inner = outer + 1; inner < 7; inner++) {
                    if (avg_press[outer] > avg_press[inner]) {
                        temp = avg_press[outer];
                        avg_press[outer] = avg_press[inner];
                        avg_press[inner] = temp;
                    }
                }
            }

            unsigned long long D1 = (avg_press[2] + avg_press[2] + avg_press[4] + avg_press[5]) / 4;

            P = ((D1 * SENS >> 21) - OFF) >> 15;


            char pressure[25];
            sprintf(pressure, "%.2f", (float) P / 100.0);
            UG_PutString(15, 20, pressure);
            put_sh1106();
            UG_PutString(75, 20, "mbar");


            put_sh1106();

            float alt = ((powf(source / ((float) P / 100.0), 0.19022256) - 1.0) * ((float) TEMP / 100 + 273.15)) / 0.0065;


            char *tmpSign = (alt < 0) ? "-" : "";
            float tmpVal = (alt < 0) ? -alt : alt;

            int tmpInt1 = tmpVal;
            float tmpFrac = tmpVal - tmpInt1;
            int tmpInt2 = trunc(tmpFrac * 100);


            char altitude[25];
            sprintf(altitude, "%s%d.%02d       ", tmpSign, tmpInt1, tmpInt2);
            UG_PutString(15, 50, altitude);
            UG_PutString(75, 50, "alt");
            put_sh1106();

            
            count_press = 0;

        }



        if (PORTBbits.RB0 == 0) {
            delay_ms(20);
            if (PORTBbits.RB0 == 0) {
                source = (float) P / 100.0;
            }

        }


        ms5611_run(x_offset, y_offset, rad);


        count_temp++;
        count_press++;

    }
}
