
#include <stdint.h>
#include "i2c.h"
#include "sh1106.h"
#include <string.h>



static unsigned char buffer[128 * 8];

void lcd_init(void) {

    write_cmd(SH1106_DISPLAYOFF);
    write_cmd(SH1106_SETDISPLAYCLOCKDIV);
    write_cmd(0x80);
    write_cmd(SH1106_SETMULTIPLEX);
    write_cmd(0x3F);
    write_cmd(SH1106_SETDISPLAYOFFSET);
    write_cmd(0x00);
    write_cmd(SH1106_SETSTARTLINE | 0x0);
    write_cmd(SH1106_MEMORYMODE);
    write_cmd(0x00);
    write_cmd(SH1106_SEGREMAP | 0x1);
    write_cmd(SH1106_COMSCANDEC);
    write_cmd(SH1106_SETCOMPINS);
    write_cmd(0x12);
    write_cmd(SH1106_SETCONTRAST);
    write_cmd(0xCF);
//    write_cmd(0x02);
    write_cmd(SH1106_SETPRECHARGE);
    write_cmd(0xF1);
    write_cmd(SH1106_SETVCOMDETECT);
    write_cmd(0x40);
    write_cmd(SH1106_DISPLAYALLON_RESUME);
    write_cmd(SH1106_NORMALDISPLAY);
    write_cmd(SH1106_DISPLAYON);


    write_cmd(SH1106_SETLOWCOLUMN | 0x0);
    write_cmd(SH1106_SETHIGHCOLUMN | 0x0);
    write_cmd(SH1106_SETSTARTLINE | 0x0);

}

void write_cmd(uint8_t command) {
    StartI2C();
    IdleI2C();
    WriteI2C(SH1106_ADDRESS);
    WriteI2C(0x00);
    WriteI2C(command);
    StopI2C();
}

void put_oled(uint8_t *buffer) {

        char height = 64;
        char width = 128;
        char m_row = 0;
        char m_col = 2;
        char j;
        char i;
        char k;
        int p = 0;
    
        height >>= 3;
        width >>= 3;
    
        for (i = 0; i < height; i++) {
            write_cmd(0xB0 + i + m_row);
            write_cmd(m_col & 0xf);
            write_cmd(0x10 | (m_col >> 4));
    
            for (j = 0; j < 8; j++) {
                StartI2C();
                IdleI2C();
                WriteI2C(SH1106_ADDRESS);
                WriteI2C(0x40);
                for (k = 0; k < width; k++, p++) {
                    WriteI2C(buffer[p]);
                }
                StopI2C();
            }
        }
}

void put_sh1106() {
    put_oled(buffer);
}

void pset(UG_S16 x, UG_S16 y, UG_COLOR c) {
    unsigned int i, p;

    if (x > 127) return;
    p = y >> 3;
    p = p << 7;
    p += x;

    if (c) {
        buffer[p] |= 1 << (y % 8);
    } else {
        buffer[p] &= ~(1 << (y % 8));
    }
}

void clearDisplay(void) {
    memset(buffer, 0, (SH1106_LCDWIDTH * SH1106_LCDHEIGHT / 8));
}

void invertDisplay(uint8_t i) {
    if (i) {
        write_cmd(SH1106_INVERTDISPLAY);
    } else {
        write_cmd(SH1106_NORMALDISPLAY);
    }
}

void startscrollright(uint8_t start, uint8_t stop) {
    write_cmd(SH1106_RIGHT_HORIZONTAL_SCROLL);
    write_cmd(0X00);
    write_cmd(start);
    write_cmd(0X00);
    write_cmd(stop);
    write_cmd(0X00);
    write_cmd(0XFF);
    write_cmd(SH1106_ACTIVATE_SCROLL);
}

void startscrollleft(uint8_t start, uint8_t stop) {
    write_cmd(SH1106_LEFT_HORIZONTAL_SCROLL);
    write_cmd(0X00);
    write_cmd(start);
    write_cmd(0X00);
    write_cmd(stop);
    write_cmd(0X00);
    write_cmd(0XFF);
    write_cmd(SH1106_ACTIVATE_SCROLL);
}

void startscrolldiagright(uint8_t start, uint8_t stop) {
    write_cmd(SH1106_SET_VERTICAL_SCROLL_AREA);
    write_cmd(0X00);
    write_cmd(SH1106_LCDHEIGHT);
    write_cmd(SH1106_VERTICAL_AND_RIGHT_HORIZONTAL_SCROLL);
    write_cmd(0X00);
    write_cmd(start);
    write_cmd(0X00);
    write_cmd(stop);
    write_cmd(0X01);
    write_cmd(SH1106_ACTIVATE_SCROLL);
}

void startscrolldiagleft(uint8_t start, uint8_t stop) {
    write_cmd(SH1106_SET_VERTICAL_SCROLL_AREA);
    write_cmd(0X00);
    write_cmd(SH1106_LCDHEIGHT);
    write_cmd(SH1106_VERTICAL_AND_LEFT_HORIZONTAL_SCROLL);
    write_cmd(0X00);
    write_cmd(start);
    write_cmd(0X00);
    write_cmd(stop);
    write_cmd(0X01);
    write_cmd(SH1106_ACTIVATE_SCROLL);
}

void stopscroll(void) {
    write_cmd(SH1106_DEACTIVATE_SCROLL);
}




